@@ -1,168 +1,175 @@
 package HTML::Make;
 use warnings;
 use strict;
-our $VERSION = 0.04;
+our $VERSION = 0.05;
 use Carp;
+use HTML::Valid::Tagset ':all';
 
 # This is a list of valid tags.
 
+my %tags = %HTML::Valid::Tagset::isKnown;
+
+# Up to version 0.04 the following list was used. This will probably
+# be removed in later versions of this module.
+
 # Extracted from
 
 # https://developer.mozilla.org/en-US/docs/Web/HTML/Element?redirectlocale=en-US&redirectslug=HTML%2FElement
 
 # by "get-tags.pl".
 
-my %tags = qw/
-a 1
-abbr 1
-acronym 1
-address 1
-applet 1
-area 1
-article 1
-aside 1
-audio 1
-b 1
-base 1
-basefont 1
-bdi 1
-bdo 1
-bgsound 1
-big 1
-blink 1
-blockquote 1
-body 1
-br 1
-button 1
-canvas 1
-caption 1
-center 1
-cite 1
-code 1
-col 1
-colgroup 1
-content 1
-data 1
-datalist 1
-dd 1
-decorator 1
-del 1
-details 1
-dfn 1
-dir 1
-div 1
-dl 1
-dt 1
-element 1
-em 1
-embed 1
-fieldset 1
-figcaption 1
-figure 1
-font 1
-footer 1
-form 1
-frame 1
-frameset 1
-h1 1
-h2 1
-h3 1
-h4 1
-h5 1
-h6 1
-head 1
-header 1
-hgroup 1
-hr 1
-html 1
-i 1
-iframe 1
-img 1
-input 1
-ins 1
-isindex 1
-kbd 1
-keygen 1
-label 1
-legend 1
-li 1
-link 1
-listing 1
-main 1
-map 1
-mark 1
-marquee 1
-menu 1
-menuitem 1
-meta 1
-meter 1
-nav 1
-nobr 1
-noframes 1
-noscript 1
-object 1
-ol 1
-optgroup 1
-option 1
-output 1
-p 1
-param 1
-plaintext 1
-pre 1
-progress 1
-q 1
-rp 1
-rt 1
-ruby 1
-s 1
-samp 1
-script 1
-section 1
-select 1
-shadow 1
-small 1
-source 1
-spacer 1
-span 1
-strike 1
-strong 1
-style 1
-sub 1
-summary 1
-sup 1
-table 1
-tbody 1
-td 1
-template 1
-textarea 1
-tfoot 1
-th 1
-thead 1
-time 1
-title 1
-tr 1
-track 1
-tt 1
-u 1
-ul 1
-var 1
-video 1
-wbr 1
-xmp 1
-/;
+# qw/
+# a 1
+# abbr 1
+# acronym 1
+# address 1
+# applet 1
+# area 1
+# article 1
+# aside 1
+# audio 1
+# b 1
+# base 1
+# basefont 1
+# bdi 1
+# bdo 1
+# bgsound 1
+# big 1
+# blink 1
+# blockquote 1
+# body 1
+# br 1
+# button 1
+# canvas 1
+# caption 1
+# center 1
+# cite 1
+# code 1
+# col 1
+# colgroup 1
+# content 1
+# data 1
+# datalist 1
+# dd 1
+# decorator 1
+# del 1
+# details 1
+# dfn 1
+# dir 1
+# div 1
+# dl 1
+# dt 1
+# element 1
+# em 1
+# embed 1
+# fieldset 1
+# figcaption 1
+# figure 1
+# font 1
+# footer 1
+# form 1
+# frame 1
+# frameset 1
+# h1 1
+# h2 1
+# h3 1
+# h4 1
+# h5 1
+# h6 1
+# head 1
+# header 1
+# hgroup 1
+# hr 1
+# html 1
+# i 1
+# iframe 1
+# img 1
+# input 1
+# ins 1
+# isindex 1
+# kbd 1
+# keygen 1
+# label 1
+# legend 1
+# li 1
+# link 1
+# listing 1
+# main 1
+# map 1
+# mark 1
+# marquee 1
+# menu 1
+# menuitem 1
+# meta 1
+# meter 1
+# nav 1
+# nobr 1
+# noframes 1
+# noscript 1
+# object 1
+# ol 1
+# optgroup 1
+# option 1
+# output 1
+# p 1
+# param 1
+# plaintext 1
+# pre 1
+# progress 1
+# q 1
+# rp 1
+# rt 1
+# ruby 1
+# s 1
+# samp 1
+# script 1
+# section 1
+# select 1
+# shadow 1
+# small 1
+# source 1
+# spacer 1
+# span 1
+# strike 1
+# strong 1
+# style 1
+# sub 1
+# summary 1
+# sup 1
+# table 1
+# tbody 1
+# td 1
+# template 1
+# textarea 1
+# tfoot 1
+# th 1
+# thead 1
+# time 1
+# title 1
+# tr 1
+# track 1
+# tt 1
+# u 1
+# ul 1
+# var 1
+# video 1
+# wbr 1
+# xmp 1
+# /;
 
-my %noCloseTags = (
-	"area" => 1,
-	"br" => 1,
-	"dd" => 1,
-	"dt" => 1,
-	"hr" => 1,
-	"image" => 1,
-	"input" => 1,
-	"img" => 1,
-	"link" => 1,
-	"meta" => 1,
-);
+my %noCloseTags = %HTML::Valid::Tagset::emptyElement;
+# (
+# 	"area" => 1,
+# 	"br" => 1,
+# 	"dd" => 1,
+# 	"dt" => 1,
+# 	"hr" => 1,
+# 	"image" => 1,
+# 	"input" => 1,
+# 	"img" => 1,
+# 	"link" => 1,
+# 	"meta" => 1,
+# );
 
 our $texttype = 'text';
 our $blanktype = 'blank';
@@ -198,6 +205,9 @@ sub new
 	if (! $options{nocheck} && $type ne $blanktype && ! $tags{lc $type}) {
 	    carp "Unknown tag type '$type'";
 	}
+	elsif (! $options{nocheck} && ! $isHTML5{lc $type}) {
+	    carp "<$type> is not HTML5";
+	}
 	if ($options{text}) {
             $obj->add_text ($options{text});
         }
@@ -223,12 +233,20 @@ sub check_attributes
 	    carp "ID attributes cannot contain spaces";
 	}
     }
+    for my $k (keys %attr) {
+	my $type = lc $obj->{type};
+	if (! tag_attr_ok (lc $type, $k)) {
+	    carp "attribute $k is not allowed for <$type> in HTML5";
+	}
+    }
 }
 
 sub add_attr
 {
     my ($obj, %attr) = @_;
-    check_attributes ($obj, %attr);
+    if (! $obj->{nocheck}) {
+	check_attributes ($obj, %attr);
+    }
     for my $k (sort keys %attr) {
 	if ($obj->{attr}->{$k}) {
 	    carp "Overwriting attribute '$k' for '$obj->{type}' tag";
